/*****************************************************************************
 * davinci.h: decoder and encoder modules using the DaVinci DSP.
 *****************************************************************************
 * Copyright (C) 2008-2009 M2X BV
 * $Id: d9d1a24b321468f49c465bd12e622e59e8a4c69e $
 *
 * Authors: Antoine Cellerier <dionoea at videolan dot org>
 *          Jean-Paul Saman <jean-paul.saman at m2x dot nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

#ifndef VLC_DAVINCI_H
#define VLC_DAVINCI_H

#define XDM_INCLUDE_DOT9_SUPPORT /* Support old 0.9 API */
#include <gnu/targets/std.h>
#include <xdc/std.h>
#include <ti/xdais/xdas.h>
#include <ti/xdais/dm/xdm.h>
#include <ti/sdo/ce/CERuntime.h>
#include <ti/sdo/ce/Engine.h>

#undef DEBUG_DAVINCI

#define ENC_CFG_PREFIX "sout-davinci-"
#define DEC_CFG_PREFIX "davinci-"

int OpenVideoDecoder( vlc_object_t * );
void CloseVideoDecoder( vlc_object_t * );

int OpenAudioDecoder( vlc_object_t * );
void CloseAudioDecoder( vlc_object_t * );

int OpenVideoEncoder( vlc_object_t * );
void CloseVideoEncoder( vlc_object_t * );

/*****************************************************************************
 * Common stuff
 *****************************************************************************/
extern const char *ppsz_engine_error[];

int davinci_AllocateBuffer( XDAS_Int32, XDAS_Int32 *, XDM_BufDesc * );
void davinci_FreeBuffer( XDM_BufDesc * );
const char *davinci_GetExtendedError( XDAS_Int32 );
void davinci_PrintAvailableAlgorithms( vlc_object_t *, const char * );

bool GetDavinciDecoder( vlc_fourcc_t, int *, const char **, const char ** );
bool GetDavinciEncoder( vlc_fourcc_t, int *, const char **, const char ** );
bool GetVlcFourcc( const char *, int *, vlc_fourcc_t *, const char ** );
XDAS_Int32 VlcChromaToXdm( vlc_fourcc_t );

#endif
