/*****************************************************************************
 * fourcc.c: davinci <-> libvlc conversion routines
 *****************************************************************************
 * Copyright (C) 2009 M2X BV
 * $Id: 1c275fef081732a731ab81326ed4b3f7795c180d $
 *
 * Authors: Jean-Paul Saman <jpsaman@videolan.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <vlc_common.h>
#include <vlc_codec.h>

#include "davinci.h"

/*****************************************************************************
 * Codec fourcc -> davinic_id mapping
 *****************************************************************************/

/* Enable codecs by changing #undef in #define here below */
#undef DAVINCI_HAS_MPEG1_CODEC

#define DAVINCI_HAS_MPEG2_CODEC
#define DAVINCI_HAS_MPEG4_CODEC
#define DAVINCI_HAS_H264_CODEC
#define DAVINCI_HAS_VC1_CODEC

#undef DAVINCI_HAS_MP3_CODEC
#undef DAVINCI_HAS_AAC_CODEC

/* decoders */
#define DAVINCI_DEC_MPEG1 "mpegdec"
#define DAVINCI_DEC_MPEG2 "mpeg2dec"
#define DAVINCI_DEC_MPEG4 "mpeg4dec"
#define DAVINCI_DEC_H264  "h264dec"
#define DAVINCI_DEC_VC1   "vc1dec"

#define DAVINIC_DEC_MP3    "mp3dec"
#define DAVINCI_DEC_AAC    "aacdec"

/* encoders */
#define DAVINCI_ENC_MPEG1 "mpegenc"
#define DAVINCI_ENC_MPEG2 "mpeg2enc"
#define DAVINCI_ENC_MPEG4 "mpeg4enc"
#define DAVINCI_ENC_H264  "h264enc"
#define DAVINCI_ENC_VC1   "vc1enc"

#define DAVINIC_NONE      ""

static const struct
{
    vlc_fourcc_t  i_fourcc;
    const char psz_decoder[10];
    const char psz_encoder[10];
    int  i_cat;
    const char psz_name[36];
} codecs_table[] =
{
    /*
     * Video Codecs
     */
#ifdef DAVINCI_HAS_MPEG1_CODEC
    /* MPEG-1 Video */
    { VLC_FOURCC('m','p','1','v'), DAVINCI_DEC_MPEG1, DAVINCI_ENC_MPEG1,
      VIDEO_ES, "MPEG-1 Video" },
    { VLC_FOURCC('m','p','e','g'), DAVINCI_DEC_MPEG1, DAVINCI_ENC_MPEG1,
      VIDEO_ES, "MPEG-1 Video" },
    { VLC_FOURCC('m','p','g','1'), DAVINCI_DEC_MPEG1, DAVINCI_ENC_MPEG1,
      VIDEO_ES, "MPEG-1 Video" },
    { VLC_FOURCC('P','I','M','1'), DAVINCI_DEC_MPEG1, DAVINCI_ENC_MPEG1,
      VIDEO_ES, "Pinnacle DC1000 (MPEG-1 Video)" },
#endif
#ifdef DAVINCI_HAS_MPEG2_CODEC
    /* MPEG-2 Video */
    { VLC_FOURCC('m','p','2','v'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "MPEG-2 Video" },
    { VLC_FOURCC('M','P','E','G'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "MPEG-2 Video" },
    { VLC_FOURCC('m','p','g','v'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "MPEG-2 Video" },
    { VLC_FOURCC('m','p','g','2'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "MPEG-2 Video" },
    { VLC_FOURCC('h','d','v','1'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "HDV 720p30 (MPEG-2 Video)" },
    { VLC_FOURCC('h','d','v','2'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "Sony HDV (MPEG-2 Video)" },
    { VLC_FOURCC('h','d','v','3'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "FCP HDV (MPEG-2 Video)" },
    { VLC_FOURCC('h','d','v','5'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "HDV 720p25 (MPEG-2 Video)" },
    { VLC_FOURCC('h','d','v','6'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "HDV 1080p24 (MPEG-2 Video)" },
    { VLC_FOURCC('h','d','v','7'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "HDV 1080p25 (MPEG-2 Video)" },
    { VLC_FOURCC('h','d','v','8'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "HDV 1080p30 (MPEG-2 Video)" },

    { VLC_FOURCC('m','x','5','n'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "MPEG2 IMX NTSC 525/60 50mb/s (FCP)" },
    { VLC_FOURCC('m','x','5','p'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "MPEG2 IMX PAL 625/60 50mb/s (FCP)" },
    { VLC_FOURCC('m','x','4','n'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "MPEG2 IMX NTSC 525/60 40mb/s (FCP)" },
    { VLC_FOURCC('m','x','4','p'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "MPEG2 IMX PAL 625/50 40mb/s (FCP)" },
    { VLC_FOURCC('m','x','3','n'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "MPEG2 IMX NTSC 525/60 30mb/s (FCP)" },
    { VLC_FOURCC('m','x','3','p'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "MPEG2 IMX NTSC 625/50 30mb/s (FCP)" },
    { VLC_FOURCC('x','d','v','2'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "XDCAM HD 1080i60" },
    { VLC_FOURCC('A','V','m','p'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "AVID IMX PAL" },
    /* ATI VCR2 */
    { VLC_FOURCC('V','C','R','2'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "ATI VCR2 Video" },
    { VLC_FOURCC('M','M','E','S'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "Matrox MPEG-2" },
    { VLC_FOURCC('m','m','e','s'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "Matrox MPEG-2" },
    { VLC_FOURCC('P','I','M','2'), DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2,
      VIDEO_ES, "Pinnacle DC1000 (MPEG-2 Video)" },
#endif
#ifdef DAVINCI_HAS_MPEG4_CODEC
    /* MPEG-4 Video */
    { VLC_FOURCC('D','I','V','X'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('d','i','v','x'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('M','P','4','S'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('m','p','4','s'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('M','4','S','2'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('m','4','s','2'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    /* XVID flavours */
    { VLC_FOURCC('x','v','i','d'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('X','V','I','D'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('X','v','i','D'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('X','V','I','X'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('x','v','i','x'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    /* DX50 */
    { VLC_FOURCC('D','X','5','0'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('d','x','5','0'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('B','L','Z','0'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "Blizzard MPEG-4 Video" },
    { VLC_FOURCC('D','X','G','M'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "Electronic Arts Game MPEG-4 Video" },
    { VLC_FOURCC('m','p','4','v'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('M','P','4','V'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC( 4,  0,  0,  0 ), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('m','4','c','c'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('M','4','C','C'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('F','M','P','4'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('f','m','p','4'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    /* 3ivx delta 3.5 Unsupported
     * putting it here gives extreme distorted images
    { VLC_FOURCC('3','I','V','1'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('3','i','v','1'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" }, */
    /* 3ivx delta 4 */
    { VLC_FOURCC('3','I','V','2'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('3','i','v','2'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('U','M','P','4'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "UB MPEG-4 Video" },
    { VLC_FOURCC('W','V','1','F'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('S','E','D','G'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "Samsung MPEG-4 Video" },
    { VLC_FOURCC('R','M','P','4'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "REALmagic MPEG-4 Video" },
    { VLC_FOURCC('H','D','X','4'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "Jomigo HDX4 (MPEG-4 Video)" },
    { VLC_FOURCC('h','d','x','4'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "Jomigo HDX4 (MPEG-4 Video)" },
    { VLC_FOURCC('S','M','P','4'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "Samsung SMP4 (MPEG-4 Video)" },
    { VLC_FOURCC('s','m','p','4'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "Samsung SMP4 (MPEG-4 Video)" },
    { VLC_FOURCC('f','v','f','w'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "FFmpeg MPEG-4" },
    { VLC_FOURCC('F','V','F','W'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "FFmpeg MPEG-4" },
    { VLC_FOURCC('F','F','D','S'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "FFDShow MPEG-4" },
    { VLC_FOURCC('V','I','D','M'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "vidm 4.01 codec" },
    { VLC_FOURCC('D','C','O','D'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('f','m','p','4'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('M','V','X','M'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('P','M','4','V'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('f','m','p','4'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('M','4','T','3'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('G','E','O','X'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('D','M','K','2'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('D','I','G','I'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('I','N','M','C'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('S','N','4','0'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },
    { VLC_FOURCC('E','P','H','V'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MPEG-4 Video" },

    /* MSMPEG4 v1 */
    { VLC_FOURCC('D','I','V','1'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v1" },
    { VLC_FOURCC('d','i','v','1'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v1" },
    { VLC_FOURCC('M','P','G','4'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v1" },
    { VLC_FOURCC('m','p','g','4'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v1" },
    { VLC_FOURCC('m','p','4','1'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v1" },

    /* MSMPEG4 v2 */
    { VLC_FOURCC('D','I','V','2'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v2" },
    { VLC_FOURCC('d','i','v','2'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v2" },
    { VLC_FOURCC('M','P','4','2'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v2" },
    { VLC_FOURCC('m','p','4','2'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v2" },

    /* MSMPEG4 v3 / M$ mpeg4 v3 */
    { VLC_FOURCC('M','P','G','3'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
    { VLC_FOURCC('m','p','g','3'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
    { VLC_FOURCC('d','i','v','3'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
    { VLC_FOURCC('M','P','4','3'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
    { VLC_FOURCC('m','p','4','3'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
    /* DivX 3.20 */
    { VLC_FOURCC('D','I','V','3'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
    { VLC_FOURCC('D','I','V','4'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
    { VLC_FOURCC('d','i','v','4'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
    { VLC_FOURCC('D','I','V','5'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
    { VLC_FOURCC('d','i','v','5'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
    { VLC_FOURCC('D','I','V','6'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
    { VLC_FOURCC('d','i','v','6'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
    /* Cool Codec */
    { VLC_FOURCC('C','O','L','1'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
    { VLC_FOURCC('c','o','l','1'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
    { VLC_FOURCC('C','O','L','0'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
    { VLC_FOURCC('c','o','l','0'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
    /* AngelPotion stuff */
    { VLC_FOURCC('A','P','4','1'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
    /* 3ivx doctered divx files */
    { VLC_FOURCC('3','I','V','D'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
    { VLC_FOURCC('3','i','v','d'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
    /* who knows? */
    { VLC_FOURCC('3','V','I','D'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
    { VLC_FOURCC('3','v','i','d'), DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4,
      VIDEO_ES, "MS MPEG-4 Video v3" },
#endif
#ifdef DAVINCI_HAS_H264_CODEC
    /* h264 */
    { VLC_FOURCC('h','2','6','4'), DAVINCI_DEC_H264, DAVINCI_ENC_H264,
      VIDEO_ES, "H264 - MPEG-4 AVC (part 10)" },
    { VLC_FOURCC('H','2','6','4'), DAVINCI_DEC_H264, DAVINCI_ENC_H264,
      VIDEO_ES, "H264 - MPEG-4 AVC (part 10)" },
    { VLC_FOURCC('x','2','6','4'), DAVINCI_DEC_H264, DAVINCI_ENC_H264,
      VIDEO_ES, "H264 - MPEG-4 AVC (part 10)" },
    { VLC_FOURCC('X','2','6','4'), DAVINCI_DEC_H264, DAVINCI_ENC_H264,
      VIDEO_ES, "H264 - MPEG-4 AVC (part 10)" },
    /* avc1: special case h264 */
    { VLC_FOURCC('a','v','c','1'), DAVINCI_DEC_H264, DAVINCI_ENC_H264,
      VIDEO_ES, "H264 - MPEG-4 AVC (part 10)" },
    { VLC_FOURCC('A','V','C','1'), DAVINCI_DEC_H264, DAVINCI_ENC_H264,
      VIDEO_ES, "H264 - MPEG-4 AVC (part 10)" },
    { VLC_FOURCC('V','S','S','H'), DAVINCI_DEC_H264, DAVINCI_ENC_H264,
      VIDEO_ES, "Vanguard VSS H264" },
    { VLC_FOURCC('V','S','S','W'), DAVINCI_DEC_H264, DAVINCI_ENC_H264,
      VIDEO_ES, "Vanguard VSS H264" },
    { VLC_FOURCC('v','s','s','h'), DAVINCI_DEC_H264, DAVINCI_ENC_H264,
      VIDEO_ES, "Vanguard VSS H264" },
    { VLC_FOURCC('D','A','V','C'), DAVINCI_DEC_H264, DAVINCI_ENC_H264,
      VIDEO_ES, "Dicas MPEGable H.264/MPEG-4 AVC" },
    { VLC_FOURCC('d','a','v','c'), DAVINCI_DEC_H264, DAVINCI_ENC_H264,
      VIDEO_ES, "Dicas MPEGable H.264/MPEG-4 AVC" },
#endif
#ifdef DAVINCI_HAS_VC1_CODEC
    /* vc-1 */
    { VLC_FOURCC('W','V','C','1'), DAVINCI_DEC_VC1, DAVINCI_ENC_VC1,
      VIDEO_ES, "Windows Media Video VC1" },
    { VLC_FOURCC('w','v','c','1'), DAVINCI_DEC_VC1, DAVINCI_ENC_VC1,
      VIDEO_ES, "Windows Media Video VC1" },
    { VLC_FOURCC('v','c','-','1'), DAVINCI_DEC_VC1, DAVINCI_ENC_VC1,
      VIDEO_ES, "Windows Media Video VC1" },
    { VLC_FOURCC('V','C','-','1'), DAVINCI_DEC_VC1, DAVINCI_ENC_VC1,
      VIDEO_ES, "Windows Media Video VC1" },
    { VLC_FOURCC('W','M','V','1'), DAVINCI_DEC_VC1, DAVINCI_ENC_VC1,
      VIDEO_ES, "Windows Media Video VC1" },
    /* WMVA is the VC-1 codec before the standardization proces,
       it is not bitstream compatible and deprecated  */
    { VLC_FOURCC('W','M','V','A'), DAVINCI_DEC_VC1, DAVINCI_ENC_VC1,
      VIDEO_ES, "Windows Media Video Advanced Profile" },
#endif

    /*
     * Audio Codecs
     */
#ifdef DAVINCI_HAS_MP3_CODEC
    /* MPEG Audio layer 1/2/3 */
    { VLC_FOURCC('m','p','g','a'), DAVINIC_DEC_MP3, DAVINIC_NONE,
      AUDIO_ES, "MPEG Audio layer 1/2" },
    { VLC_FOURCC('m','p','3',' '), DAVINIC_DEC_MP3, DAVINIC_NONE,
      AUDIO_ES, "MPEG Audio layer 1/2/3" },
    { VLC_FOURCC('.','m','p','3'), DAVINIC_DEC_MP3, DAVINIC_NONE,
      AUDIO_ES, "MPEG Audio layer 1/2/3" },
    { VLC_FOURCC('M','P','3',' '), DAVINIC_DEC_MP3, DAVINIC_NONE,
      AUDIO_ES, "MPEG Audio layer 1/2/3" },
    { VLC_FOURCC('L','A','M','E'), DAVINIC_DEC_MP3, DAVINIC_NONE,
      AUDIO_ES, "MPEG Audio layer 1/2/3" },
#endif
#ifdef DAVINCI_HAS_AAC_CODEC
    /* AAC audio */
    { VLC_FOURCC('m','p','4','a'), DAVINCI_DEC_AAC, DAVINIC_NONE,
      AUDIO_ES, "MPEG AAC Audio" },
    { VLC_FOURCC('a','a','c',' '), DAVINCI_DEC_AAC, DAVINIC_NONE,
      AUDIO_ES, "MPEG AAC Audio" },
#endif
    { 0, "", "", 0, "" }
};

bool GetDavinciDecoder( vlc_fourcc_t i_fourcc, int *pi_cat,
                    const char **ppsz_codec, const char **ppsz_name )
{
    for( unsigned i = 0; codecs_table[i].i_fourcc != 0; i++ )
    {
        if( codecs_table[i].i_fourcc == i_fourcc )
        {
            if( pi_cat ) *pi_cat = codecs_table[i].i_cat;
            if( ppsz_codec ) *ppsz_codec = (char *)codecs_table[i].psz_decoder;
            if( ppsz_name ) *ppsz_name = (char *)codecs_table[i].psz_name;

            return true;
        }
    }
    return false;
}

bool GetDavinciEncoder( vlc_fourcc_t i_fourcc, int *pi_cat,
                    const char **ppsz_codec, const char **ppsz_name )
{
    for( unsigned i = 0; codecs_table[i].i_fourcc != 0; i++ )
    {
        if( codecs_table[i].i_fourcc == i_fourcc )
        {
            if( pi_cat ) *pi_cat = codecs_table[i].i_cat;
            if( ppsz_codec ) *ppsz_codec = (char *)codecs_table[i].psz_encoder;
            if( ppsz_name ) *ppsz_name = (char *)codecs_table[i].psz_name;

            return true;
        }
    }
    return false;
}

bool GetVlcFourcc( const char *ppsz_codec, int *pi_cat,
                   vlc_fourcc_t *pi_fourcc, const char **ppsz_name )
{
    for( unsigned i = 0; codecs_table[i].i_fourcc != 0; i++ )
    {
        if( strncmp( codecs_table[i].psz_decoder, ppsz_codec, 10 ) == 0 )
        {
            if( pi_cat ) *pi_cat = codecs_table[i].i_cat;
            if( pi_fourcc ) *pi_fourcc = codecs_table[i].i_fourcc;
            if( ppsz_name ) *ppsz_name = codecs_table[i].psz_name;

            return true;
        }
        else if( strncmp( codecs_table[i].psz_encoder, ppsz_codec, 10 ) == 0 )
        {
            if( pi_cat ) *pi_cat = codecs_table[i].i_cat;
            if( pi_fourcc ) *pi_fourcc = codecs_table[i].i_fourcc;
            if( ppsz_name ) *ppsz_name = codecs_table[i].psz_name;

            return true;
        }
    }
    return false;
}

XDAS_Int32 VlcChromaToXdm( vlc_fourcc_t i_chroma )
{
    switch( i_chroma )
    {
        case VLC_FOURCC('I','4','2','0'):
            return XDM_YUV_420P;

        case VLC_FOURCC('I','4','1','1'):
            return XDM_YUV_411P;

        case VLC_FOURCC('I','4','2','2'):
            return XDM_YUV_422P;

        case VLC_FOURCC('I','4','4','4'):
            return XDM_YUV_444P;

        case VLC_FOURCC('Y','U','Y','V'):
            return XDM_YUV_422IBE; /* FIXME ? */

        case VLC_FOURCC('U','Y','V','Y'):
            return XDM_YUV_422ILE; /* FIXME ? */

        case VLC_FOURCC('G','R','E','Y'):
            return XDM_GRAY;

#if 0
        /* FIXME: not sure about this one */
        case VLC_FOURCC('R','V','2','4'):
        case VLC_FOURCC('R','V','3','2'):
            return XDM_RGB;
#endif

        default:
            return XDM_CHROMA_NA;
    }
}
