/*****************************************************************************
 * resizer.c: Resize pictures directly into framebuffer (DaVinci specific)
 *****************************************************************************
 * Copyright (C) 2008-2009 M2X BV
 *
 * Authors: Rafaël Carré <rcarre@m2x.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

#ifndef RESIZER_H
#define RESIZER_H

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <linux/fb.h>
#include <video/davincifb.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <ti/sdo/ce/osal/Memory.h>
#include <inttypes.h>
#include <asm-arm/arch-davinci/davinci_resizer.h>
#include <math.h>

#define NUM_BUFFERS 1 /* XXX: mesure performance with 3 buffers if needed */

#define BPP 16

#define PI 3.1415926535897932384626

typedef struct
{
    int i_fd;
    struct fb_var_screeninfo var_info;
    struct fb_fix_screeninfo fix_info;
    uint8_t *p_map;
    UInt32 p_physbufs[NUM_BUFFERS];
} davinci_fb_t;

typedef struct
{
    int i_fd;
    unsigned int i_height;
    unsigned int i_width;

    uint8_t *p_yuyv;
    int     offset;
    int     i_yuyv;

    unsigned int    i_out_width;
    unsigned int    i_out_height;

    bool    b_direct;
} davinci_resizer_t;

void Resize( decoder_t *, bool, davinci_resizer_t *, davinci_fb_t *,
    XDM_BufDesc );
int DavinciInit( decoder_t *, davinci_fb_t *, davinci_resizer_t * );
void DavinciClose( davinci_fb_t *, davinci_resizer_t * );

#endif /* RESIZER_H */
